@@ -14,6 +14,8 @@ extern int Android_ScreenH;
 extern int Android_Aspect;
 extern int Android_Bilinear;
 extern int Android_CropScreen[];
+extern int Android_PortPad;
+extern int Android_CovlHold;
 extern float Android_Joyscale;
 
 int  Android_InitGraphics(void);
