@@ -2,9 +2,11 @@
  Joystick configuration utility for David Firth's Atari800 emulator
  22.5.1998 Robert Golias   (golias@fi.muni.cz)
  ******************************************************************/
+#include "config.h"
 #include <stdio.h>
 #include <stdarg.h>
-#include <strings.h>
+#include <stdlib.h>
+#include <string.h>
 
 #include <go32.h>
 #include <sys/farptr.h>
@@ -12,7 +14,6 @@
 #include <pc.h>
 #include <conio.h>
 
-#include "config.h"
 #include "pcjoy.h"
 
 /*color attributes*/
@@ -51,7 +52,7 @@ char keynames[256][11]={
       "?","?","?","?","?","?","?","?",
       "?","?","?","?","?","?","?","?",
       "?","?","?","?","?","?","?","?",
-/*keys 127 - 255 = extended scancodes (preceeded with 0xe0) */
+/*keys 127 - 255 = extended scancodes (preceded with 0xe0) */
       "?","?","?","?","?","?","?","?",
       "?","?","?","?","?","?","?","?",
       "?","?","?","?","?","?","?","?",
