@@ -1,8 +1,6 @@
 #ifndef MZPOKEYSND_H_
 #define MZPOKEYSND_H_
 
-#include "config.h"
-#include "pokeysnd.h"
 #include "atari.h"
 
 int MZPOKEYSND_Init(ULONG freq17,
@@ -15,8 +13,4 @@ int MZPOKEYSND_Init(ULONG freq17,
 #endif
                        );
 
-#ifdef SYNCHRONIZED_SOUND
-#endif /* SYNCHRONIZED_SOUND */
-int MZPOKEYSND_UpdateProcessBuffer(void);
-extern UBYTE *MZPOKEYSND_process_buffer;
 #endif /* MZPOKEYSND_H_ */
