@@ -37,6 +37,12 @@
 #endif
 #ifdef HAVE_LIBZ
 #include <zlib.h>
+#else
+#define gzFile char *
+#define Z_OK 0
+#endif
+#ifdef LIBATARI800
+#include "libatari800/init.h"
 #endif
 #ifdef DREAMCAST
 #include <bzlib/bzlib.h>
@@ -73,11 +79,11 @@
 
 #define SAVE_VERSION_NUMBER 8 /* Last changed after Atari800 3.1.0 */
 
-#if defined(MEMCOMPR)
-static gzFile *mem_open(const char *name, const char *mode);
-static int mem_close(gzFile *stream);
-static size_t mem_read(void *buf, size_t len, gzFile *stream);
-static size_t mem_write(const void *buf, size_t len, gzFile *stream);
+#if defined(MEMCOMPR) || defined(LIBATARI800)
+static gzFile mem_open(const char *name, const char *mode);
+static int mem_close(gzFile stream);
+static size_t mem_read(void *buf, size_t len, gzFile stream);
+static size_t mem_write(const void *buf, size_t len, gzFile stream);
 #define GZOPEN(X, Y)     mem_open(X, Y)
 #define GZCLOSE(X)       mem_close(X)
 #define GZREAD(X, Y, Z)  mem_read(Y, Z, X)
@@ -351,6 +357,7 @@ int StateSav_SaveAtariState(const char *filename, const char *mode, UBYTE SaveVe
 		return FALSE;
 	}
 
+	STATESAV_TAG(size);  /* initialize to 0, set to actual size if successful */
 	StateSav_SaveUBYTE(&StateVersion, 1);
 	StateSav_SaveUBYTE(&SaveVerbose, 1);
 	/* The order here is important. Atari800_StateSave must be first because it saves the machine type, and
@@ -400,6 +407,7 @@ int StateSav_SaveAtariState(const char *filename, const char *mode, UBYTE SaveVe
 	DCStateSave();
 #endif
 
+	STATESAV_TAG(size);
 	if (GZCLOSE(StateFile) != 0) {
 		StateFile = NULL;
 		return FALSE;
@@ -541,6 +549,13 @@ int StateSav_ReadAtariState(const char *filename, const char *mode)
 }
 
 
+/* Common definitions for in-memory state save used for DREAMCAST and libatari800
+ */
+#if defined(MEMCOMPR) || defined(LIBATARI800)
+static char * plainmembuf;
+static unsigned int plainmemoff;
+static unsigned int unclen;
+
 /* hack to compress in memory before writing
  * - for DREAMCAST only
  * - 2 reasons for this:
@@ -548,27 +563,21 @@ int StateSav_ReadAtariState(const char *filename, const char *mode)
  * - write in DC specific file format to provide icon and description
  */
 #ifdef MEMCOMPR
-
-static char * plainmembuf;
-static unsigned int plainmemoff;
 static char * comprmembuf;
 #define OM_READ  1
 #define OM_WRITE 2
 static int openmode;
-static unsigned int unclen;
 static char savename[FILENAME_MAX];
 #define HDR_LEN 640
 
-#define ALLOC_LEN 210000
-
 /* replacement for GZOPEN */
-static gzFile *mem_open(const char *name, const char *mode)
+static gzFile mem_open(const char *name, const char *mode)
 {
 	if (*mode == 'w') {
 		/* open for write (save) */
 		openmode = OM_WRITE;
 		strcpy(savename, name); /* remember name */
-		plainmembuf = Util_malloc(ALLOC_LEN);
+		plainmembuf = Util_malloc(STATESAV_MAX_SIZE);
 		plainmemoff = 0; /*HDR_LEN;*/
 		return (gzFile *) plainmembuf;
 	}
@@ -577,13 +586,13 @@ static gzFile *mem_open(const char *name, const char *mode)
 		FILE *f;
 		size_t len;
 		openmode = OM_READ;
-		unclen = ALLOC_LEN;
+		unclen = STATESAV_MAX_SIZE;
 		f = fopen(name, mode);
 		if (f == NULL)
 			return NULL;
-		plainmembuf = Util_malloc(ALLOC_LEN);
-		comprmembuf = Util_malloc(ALLOC_LEN);
-		len = fread(comprmembuf, 1, ALLOC_LEN, f);
+		plainmembuf = Util_malloc(STATESAV_MAX_SIZE);
+		comprmembuf = Util_malloc(STATESAV_MAX_SIZE);
+		len = fread(comprmembuf, 1, STATESAV_MAX_SIZE, f);
 		fclose(f);
 		/* XXX: does DREAMCAST's fread return ((size_t) -1) ? */
 		if (len != 0
@@ -594,7 +603,7 @@ static gzFile *mem_open(const char *name, const char *mode)
 #endif
 			free(comprmembuf);
 			plainmemoff = 0;
-			return (gzFile *) plainmembuf;
+			return (gzFile) plainmembuf;
 		}
 		free(comprmembuf);
 		free(plainmembuf);
@@ -603,16 +612,16 @@ static gzFile *mem_open(const char *name, const char *mode)
 }
 
 /* replacement for GZCLOSE */
-static int mem_close(gzFile *stream)
+static int mem_close(gzFile stream)
 {
 	int status = -1;
-	unsigned int comprlen = ALLOC_LEN - HDR_LEN;
+	unsigned int comprlen = STATESAV_MAX_SIZE - HDR_LEN;
 	if (openmode != OM_WRITE) {
 		/* was opened for read */
 		free(plainmembuf);
 		return 0;
 	}
-	comprmembuf = Util_malloc(ALLOC_LEN);
+	comprmembuf = Util_malloc(STATESAV_MAX_SIZE);
 	if (BZ2_bzBuffToBuffCompress(comprmembuf + HDR_LEN, &comprlen, plainmembuf, plainmemoff, 9, 0, 0) == BZ_OK) {
 		FILE *f;
 		f = fopen(savename, "wb");
@@ -641,9 +650,34 @@ static int mem_close(gzFile *stream)
 	free(plainmembuf);
 	return status;
 }
+#endif /* #ifdef MEMCOMPR */
+
+
+#ifdef LIBATARI800
+/* replacement for GZOPEN */
+static gzFile mem_open(const char *name, const char *mode)
+{
+	plainmembuf = (char *)LIBATARI800_StateSav_buffer;
+	plainmemoff = 0; /*HDR_LEN;*/
+	unclen = STATESAV_MAX_SIZE;
+	return (gzFile) plainmembuf;
+}
+
+/* replacement for GZCLOSE */
+static int mem_close(gzFile stream)
+{
+	return 0;
+}
+
+ULONG StateSav_Tell()
+{
+	return (ULONG)plainmemoff;
+}
+#endif /* #ifdef LIBATARI800 */
+
 
 /* replacement for GZREAD */
-static size_t mem_read(void *buf, size_t len, gzFile *stream)
+static size_t mem_read(void *buf, size_t len, gzFile stream)
 {
 	if (plainmemoff + len > unclen) return 0;  /* shouldn't happen */
 	memcpy(buf, plainmembuf + plainmemoff, len);
@@ -652,15 +686,15 @@ static size_t mem_read(void *buf, size_t len, gzFile *stream)
 }
 
 /* replacement for GZWRITE */
-static size_t mem_write(const void *buf, size_t len, gzFile *stream)
+static size_t mem_write(const void *buf, size_t len, gzFile stream)
 {
-	if (plainmemoff + len > ALLOC_LEN) return 0;  /* shouldn't happen */
+	if (plainmemoff + len > unclen) return 0;  /* shouldn't happen */
 	memcpy(plainmembuf + plainmemoff, buf, len);
 	plainmemoff += len;
 	return len;
 }
 
-#endif /* #ifdef MEMCOMPR */
+#endif /* defined(MEMCOMPR) || defined(LIBATARI800) */
 
 /*
 vim:ts=4:sw=4:
